package it.eng.spagosoa.utils;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.activation.DataHandler;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;

import org.apache.axis.message.MessageElement;
import org.apache.axis.soap.MessageFactoryImpl;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

public class SOAPUtils {

	public static SOAPMessage call(String wsEndpoint, SOAPMessage soapRequest) throws SOAPException {
		SOAPConnection soapConnection = null;
		try{
			SOAPConnectionFactory soapConnectionFactory = javax.xml.soap.SOAPConnectionFactory.newInstance();

			soapConnection = soapConnectionFactory.createConnection();

			javax.xml.soap.SOAPMessage soapResponse = soapConnection.call(soapRequest, wsEndpoint);
			return soapResponse;
		}finally{
			soapConnection.close();
		}
		

	}

	public static List<DataHandler> getAttachments(SOAPMessage soapMessage) throws SOAPException {
		@SuppressWarnings("unchecked")
		Iterator<AttachmentPart> iterator = soapMessage.getAttachments();
		List<DataHandler> attachments = new ArrayList<DataHandler>();

		while (iterator.hasNext()) {
			attachments.add(iterator.next().getDataHandler());
		}
		return attachments;
	}

	public static String getSOAPBodyAsAString(SOAPMessage soapMessage) throws SOAPException {
		SOAPBody body = soapMessage.getSOAPBody();
		/* Document doc = */body.getOwnerDocument();
		return body.toString();
	}

	public static SOAPMessage createSOAPMessageWithOneAttachment(String xmlString, DataHandler attachment)
			throws Exception {
		List<DataHandler> attachments = new ArrayList<DataHandler>();
		attachments.add(attachment);
		return createSOAPMessageWithAttachments(xmlString, attachments, "attachment", "href");
	}

	public static SOAPMessage createSOAPMessageWithOneAttachment(String xmlString, DataHandler attachment,
			String headerName, String[] xmlHeader) throws Exception {
		List<DataHandler> attachments = new ArrayList<DataHandler>();
		attachments.add(attachment);
		return createSOAPMessageWithAttachments(xmlString, attachments, "attachment", "href", headerName, xmlHeader);
	}

	public static SOAPMessage createSOAPMessageWithAttachments(String xmlString, List<DataHandler> attachments)
			throws Exception {
		return createSOAPMessageWithAttachments(xmlString, attachments, "attachment", "href");
	}

	public static SOAPMessage createSOAPMessageWithAttachments(String xmlString, List<DataHandler> attachments,
			String attachmentEnvelopeName, String attachmentRefAttributeName) throws Exception {
		return createSOAPMessageWithAttachments(xmlString, attachments, attachmentEnvelopeName, attachmentRefAttributeName,
				null, null);
	}

	public static SOAPMessage createSOAPMessageWithAttachments(String xmlString, List<DataHandler> attachments,
			String attachmentEnvelopeName, String attachmentRefAttributeName, String headerName, String[] xmlHeader)
			throws Exception {
		SOAPMessage message = new MessageFactoryImpl().createMessage(); 
		SOAPPart soap = message.getSOAPPart();
		SOAPEnvelope envelope = soap.getEnvelope();
		SOAPBody body = envelope.getBody();
		Document doc = createDOMFromString(xmlString);
		if (doc == null)
			throw new IOException("Problem reading and creating DOM from xmlString createDOMFromString returned null");
		SOAPElement bodyElement = body.addDocument(doc);

		if (headerName != null) {
			// Add a SOAP header
			SOAPHeader header = envelope.getHeader();
			Name soapHeaderName = envelope.createName(headerName);
			SOAPHeaderElement shElement = header.addHeaderElement(soapHeaderName);

			for (String singleHeader : xmlHeader) {
				Document docHeader = createDOMFromString(singleHeader);
				SOAPElement me = new MessageElement(docHeader.getDocumentElement());
				shElement.addChildElement(me);
			}
		}

		if (attachments != null) {
			for (DataHandler dh : attachments) {
				javax.xml.soap.AttachmentPart attachmentPart = message.createAttachmentPart(dh);
				message.addAttachmentPart(attachmentPart);

				// Add to the SOAP body an element containing a reference to the
				// attachment
				// like in the following sample:
				// <attachment href="cid:213297E5A6EA07762222BCF8633170B8" xmlns="" />
				javax.xml.soap.SOAPElement element = bodyElement.addChildElement(envelope.createName(attachmentEnvelopeName));
				element.addAttribute(envelope.createName(attachmentRefAttributeName), "cid:" + attachmentPart.getContentId());
			}
		}
		return message;
	}

	public static SOAPMessage createSOAPMessageFromString(String xmlString) throws Exception {
		SOAPMessage message = new MessageFactoryImpl().createMessage();
		SOAPPart soap = message.getSOAPPart();
		SOAPEnvelope envelope = soap.getEnvelope();
		SOAPBody body = envelope.getBody();
		Document doc = createDOMFromString(xmlString);
		if (doc == null)
			throw new IOException("Problem reading and creating DOM from xmlString createDOMFromString returned null");
		SOAPElement bodyElement = body.addDocument(doc);

		return message;
	}

	public static SOAPMessage createSOAPMessageFromFile(String filename) throws Exception {
		SOAPMessage message = new MessageFactoryImpl().newInstance().createMessage();
		SOAPPart soap = message.getSOAPPart();
		SOAPEnvelope envelope = soap.getEnvelope();
		SOAPBody body = envelope.getBody();
		Document doc = createDOMFromFile(filename);
		if (doc == null)
			throw new IOException("Problem reading and creating DOM from file,createDOMFromFile returned null");
		SOAPElement element = body.addDocument(doc);
		return message;
	}

	public static SOAPElement createSOAPBodyFromString(String xmlString) throws Exception {
		SOAPMessage message = new MessageFactoryImpl().createMessage();
		SOAPPart soap = message.getSOAPPart();
		SOAPEnvelope envelope = soap.getEnvelope();
		SOAPBody body = envelope.getBody();
		Document doc = createDOMFromString(xmlString);
		if (doc == null)
			throw new IOException("Problem reading and creating DOM from xmlString createDOMFromString returned null");
		SOAPElement element = body.addDocument(doc);
		return element;
	}

	public static SOAPElement createSOAPBodyFile(String filename) throws Exception {
		SOAPMessage message = new MessageFactoryImpl().createMessage();
		SOAPPart soap = message.getSOAPPart();
		SOAPEnvelope envelope = soap.getEnvelope();
		SOAPBody body = envelope.getBody();
		Document doc = createDOMFromFile(filename);
		if (doc == null)
			throw new IOException("Problem reading and creating DOM from file,createDOMFromFile returned null");
		SOAPElement element = body.addDocument(doc);
		return element;
	}

	/**
	 * Read a file and create a DOM out of it
	 * 
	 * @param filename
	 *          the file where the XML is stored
	 * @return
	 */
	public static Document createDOMFromFile(String filename) throws Exception {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setNamespaceAware(true);

		DocumentBuilder parser = factory.newDocumentBuilder();
		InputStream in = Thread.currentThread().getContextClassLoader().getResourceAsStream(filename);
		Document xml = parser.parse(in);
		in.close();

		return xml;
	}

	public static Document createDOMFromString(String xmlString) throws Exception {

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		
		
		factory.setValidating(false);
		factory.setNamespaceAware(true);

		DocumentBuilder parser = factory.newDocumentBuilder();
		Document xml = parser.parse(new InputSource(new StringReader(xmlString)));
		
		return xml;
	}
}
