package org.spagic.workflow.api.jbpm;

import java.util.List;
import java.util.Set;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Property;
import org.spagic.graph.parser.IProcessInfo;
import org.spagic.graph.parser.StepInfo;
import org.spagic.graph.parser.SubProcessInfo;
import org.spagic.metadb.base.TransitionStateConstants;
import org.spagic.metadb.base.TransitionStateUtils;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.Step;
import org.spagic.metadb.model.TransitionInstance;

public class ProcessInfo implements IProcessInfo {

	private Session aSession;

	public ProcessInfo(Session session) {
		aSession = session;
	}

	public int getExecutionStateInstanceService(String stepName, long processInstanceId) {
		int prevStatus = TransitionStateConstants.TRANSITION_MISSING;
		Step aStep = AuditDBUtils.getStepByName(aSession, stepName);
		List<TransitionInstance> transitionInstanceList = null;
		if (aStep.isIsStart()) {
			transitionInstanceList = getSourceTransitionInstance(aSession, aStep.getIdStep(), processInstanceId);
			if (transitionInstanceList != null && transitionInstanceList.size() > 0)
				return TransitionStateConstants.TRANSITION_DONE;
			else
				return TransitionStateConstants.TRANSITION_MISSING;
		}
		transitionInstanceList = getTransitionInstance(aSession, aStep.getIdStep(), processInstanceId);
		// List<TransitionInstance> transitionInstanceList =
		// getTransitionInstance(aStep.getIdStep(), processInstanceId);
		prevStatus = TransitionStateConstants.TRANSITION_MISSING;
		for (TransitionInstance transitionInstance : transitionInstanceList) {
			int state = transitionInstance.getTransitionState().getIdState();
			if (TransitionStateUtils.isFaultState(state) && prevStatus > TransitionStateConstants.TRANSITION_FAULTED)
				return TransitionStateConstants.TRANSITION_FAULTED;
			if (TransitionStateUtils.isActiveState(state) && prevStatus > TransitionStateConstants.TRANSITION_ACTIVE)
				prevStatus = TransitionStateConstants.TRANSITION_ACTIVE;
			if (TransitionStateUtils.isDoneState(state)) {
				if (transitionInstance.isRuntimeWarning() && prevStatus > TransitionStateConstants.TRANSITION_WARNING)
					prevStatus = TransitionStateConstants.TRANSITION_WARNING;
				else if (prevStatus > TransitionStateConstants.TRANSITION_DONE)
					prevStatus = TransitionStateConstants.TRANSITION_DONE;
			}
		}
		// Transition states are all negatives constants
		return prevStatus;

	}

	@SuppressWarnings("unchecked")
	private static List<TransitionInstance> getTransitionInstance(Session session, Long stepId, long processInstanceId)
			throws HibernateException {
		List<TransitionInstance> transitionInstance = null;
		Criteria criteria = session.createCriteria(TransitionInstance.class);
		criteria.add(Expression.eq("stepByIdStepTarget.idStep", stepId));
		criteria.add(Expression.eq("processInstance.idProcessInstance", processInstanceId));
		criteria.addOrder(Order.asc("transitionStart"));
		transitionInstance = criteria.list();
		return transitionInstance;
	}

	@SuppressWarnings("unchecked")
	private static List<TransitionInstance> getSourceTransitionInstance(Session session, long stepId,
			long processInstanceId) throws HibernateException {
		List<TransitionInstance> transitionInstance = null;
		Criteria criteria = session.createCriteria(TransitionInstance.class);
		criteria.add(Expression.eq("stepByIdStepSource.idStep", stepId));
		criteria.add(Expression.eq("processInstance.idProcessInstance", processInstanceId));
		criteria.addOrder(Order.asc("transitionStart"));
		transitionInstance = criteria.list();
		return transitionInstance;
	}

	public StepInfo getStepInfo(String stepName) {
		Step step = AuditDBUtils.getStepByName(aSession, stepName);
		if (step != null) {
			StepInfo stepInfo = new StepInfo(step.getIdStep().toString(), step.getName(), null, step.getState(), step.isIsStart());
			return stepInfo;
		}
		return null;
	}

	public SubProcessInfo getSubProcessInfo(String stepName) {
		Step step = AuditDBUtils.getStepByName(aSession, stepName);
		Set<PropertyStep> propertySet = step.getPropertySteps();
		for (PropertyStep propertyStep : propertySet)
			if (propertyStep.getProperty().getCode().equals("SubProcess")) {
				String processName = propertyStep.getProperty().getValue();
				DetachedCriteria lastVersion = DetachedCriteria.forClass(org.spagic.metadb.model.Process.class).add(
						Property.forName("name").eq(processName)).setProjection(Property.forName("idProcess").max());

				Criteria criteria = aSession.createCriteria(org.spagic.metadb.model.Process.class, "process").add(
						Property.forName("name").eq(processName)).add(Property.forName("idProcess").eq(lastVersion));
				org.spagic.metadb.model.Process process = (org.spagic.metadb.model.Process) criteria.uniqueResult();
				SubProcessInfo subProcess = new SubProcessInfo(process.getIdProcess(), processName);
				return subProcess;
			}
		return null;
	}

}
