/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic.workflow.command;

import java.util.HashMap;
import java.util.Map;

import org.spagic.workflow.api.IControlAPI;
import org.spagic.workflow.api.ModifiedMessage;
import org.spagic.workflow.api.Variable;

public class CMDControlAPI implements IControlAPI {

	private ICommandBasedProcessEngine engine = null;

	public CMDControlAPI(ICommandBasedProcessEngine engine) {
		this.engine =  engine;

	}

	public void abort(long processInstanceId) {
		String method = CmdObject.ABORT;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);

	}

	public void restart(long processInstanceId, ModifiedMessage[] modifiedMessages) {
		String method = CmdObject.RESTART;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("modifiedMessages", modifiedMessages);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void stopOnAutomaticTask(long processInstanceId, boolean stop) {
		String method = CmdObject.STOP_ON_AUTOMATIC_TASK;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("stopFlag", stop);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void executeTask(long processInstanceId, String taskName, Variable[] vars) {
		String method = CmdObject.EXECUTE_TASK_BY_TASK_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void executeTask(long processInstanceId, long taskId, Variable[] vars) {
		String method = CmdObject.EXECUTE_TASK_BY_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}



	public void executeTaskByActor(String actor, long processInstanceId, String taskName, Variable[] vars) {
		String method = CmdObject.EXECUTE_TASK_BY_ACTOR_AND_TASK_NAME;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actor", actor);
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void executeTaskByActor(String actor, long processInstanceId, long taskId, Variable[] vars) {
		String method = CmdObject.EXECUTE_TASK_BY_ACTOR_AND_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actor", actor);
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}
	
	public void executeTaskWithStatus(long processInstanceId, String taskName, String taskStatus, Variable[] vars) {
		String method = CmdObject.EXECUTE_TASK_WITH_STATUS_BY_TASK_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("taskStatus", taskStatus);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void executeTaskWithStatus(long processInstanceId, long taskId, String taskStatus, Variable[] vars) {
		String method = CmdObject.EXECUTE_TASK_WITH_STATUS_BY_TASK_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);
		paramMap.put("taskStatus", taskStatus);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public Variable getGlobalVariable(long processInstanceId, String variableName) {
		String method = CmdObject.GET_GLOBAL_VARIABLE;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("variableName", variableName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		Variable variable = (Variable) resultObject.getResult();

		return variable;
	}

	public Variable[] getGlobalVariables(long processInstanceId) {
		String method = CmdObject.GET_GLOBAL_VARIABLES;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		Variable[] variables = (Variable[]) resultObject.getResult();

		return variables;
	}

	public void linkProcesses(long firstProcessInstanceId, long secondProcessInstanceId) {
		String method = CmdObject.LINK_PROCESSES;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("firstProcessInstanceId", firstProcessInstanceId);
		paramMap.put("secondProcessInstanceId", secondProcessInstanceId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);

	}

	public void setGlobalVariable(long processInstanceId, String variableName, Object variableValue) {
		String method = CmdObject.SET_GLOBAL_VARIABLE;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("variableName", variableName);

		paramMap.put("variableValue", variableValue);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void setGlobalVariables(long processInstanceId, Variable[] variables) {
		String method = CmdObject.SET_GLOBAL_VARIABLES;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);

		paramMap.put("variables", variables);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public Long startByProcessName(String processName) {
		String method = CmdObject.START_BY_PROCESS_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		if (resultObject == null)
			return null;
		return (Long) resultObject.getResult();
	}

	public Variable getTaskVariable(long processInstanceId, String taskName, String variableName) {
		String method = CmdObject.GET_TASK_VARIABLE;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("variableName", variableName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		Variable variable = (Variable) resultObject.getResult();

		return variable;
	}

	public Variable[] getTaskVariables(long processInstanceId, String taskName) {
		String method = CmdObject.GET_TASK_VARIABLES;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		Variable[] variables = (Variable[]) resultObject.getResult();

		return variables;
	}

	public void setTaskVariable(long processInstanceId, String taskName, String variableName, Object variableValue) {
		String method = CmdObject.SET_TASK_VARIABLE;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("variableName", variableName);

		paramMap.put("variableValue", variableValue);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void setTaskVariables(long processInstanceId, String taskName, Variable[] variables) {
		String method = CmdObject.SET_TASK_VARIABLES;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);

		paramMap.put("variables", variables);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public Long startByProcessName(String processName, Variable[] vars) {
		String method = CmdObject.START_BY_PROCESS_NAME_AND_VARIABLES;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);
		paramMap.put("variables", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		if (resultObject == null)
			return null;
		return (Long) resultObject.getResult();
	}

	public void advanceTaskCompletion(long processInstanceId, long taskId, int completionPercentage) {
		String method = CmdObject.ADVANCE_TASK_COMPLETION_BY_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);
		paramMap.put("completionPercentage", new Integer(completionPercentage));

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void advanceTaskCompletion(long processInstanceId, String taskName, int completionPercentage) {
		String method = CmdObject.ADVANCE_TASK_COMPLETION_BY_TASK_NAME;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("completionPercentage", new Integer(completionPercentage));

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}
	
	public void advanceTaskCompletionByActor(String actorId,
			long processInstanceId, long taskId, int completionPercentage) {
		String method = CmdObject.ADVANCE_TASK_COMPLETION_BY_ACTOR_AND_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actorId", actorId);
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);
		paramMap.put("completionPercentage", new Integer(completionPercentage));

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
		
	}


	public void advanceTaskCompletionByActor(String actorId,
			long processInstanceId, String taskName, int completionPercentage) {
		String method = CmdObject.ADVANCE_TASK_COMPLETION_BY_ACTOR_AND_TASK_NAME;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actorId", actorId);
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("completionPercentage", new Integer(completionPercentage));

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
		
	}

	public void startTask(long processInstanceId, long taskId, Variable[] vars) {
		String method = CmdObject.START_TASK_BY_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void startTask(long processInstanceId, String taskName, Variable[] vars) {
		String method = CmdObject.START_TASK_BY_TASK_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void startTaskByActor(String actor, long processInstanceId, long taskId, Variable[] vars) {
		String method = CmdObject.START_TASK_BY_ACTOR_AND_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);
		paramMap.put("actor", actor);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void startTaskByActor(String actor, long processInstanceId, String taskName, Variable[] vars) {
		String method = CmdObject.START_TASK_BY_ACTOR_AND_TASK_NAME;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		paramMap.put("actor", actor);
		paramMap.put("vars", vars);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	public void rejectTask(long processInstanceId, long taskId) {
		String method = CmdObject.REJECT_TASK_BY_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
	}

	
	public void rejectTaskByActor(String actorId, long processInstanceId,long taskId) {
		String method = CmdObject.REJECT_TASK_BY_ACTOR_AND_TASK_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actorId", actorId);
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskId", taskId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		this.engine.executeCmdObject(cmdObject);
		
	}

	
	
	
	

	

}
