/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package mytest;

import java.util.Map;
import java.util.Scanner;
import java.util.StringTokenizer;

import org.slf4j.LoggerFactory;
import org.spagic.workflow.api.IControlAPI;
import org.spagic.workflow.api.IProcessEngine;
import org.spagic.workflow.api.IQueryAPI;
import org.spagic.workflow.api.Variable;
import org.spagic.workflow.api.ws.client.WSProcessEngine;

public class APITester {
	
	IProcessEngine engine = null;
	IControlAPI controlAPI = null;
	IQueryAPI queryAPI = null;
	
	
	// Id Process Instance, Process Name
	Map<Long, String> startedInstances = null;
	
	public APITester(){
		engine = new WSProcessEngine("http://localhost:9090/wfengine/");
		controlAPI = engine.getControlAPI();
		queryAPI = engine.getQueryAPI();
		
	}
	
	public void open() throws Exception {
		LoggerFactory.getLogger(this.getClass());
		try {
			String cmd = null;
		    Scanner in = new Scanner(System.in);

		    while (true){
		    	System.out.print(" -> "); 
		    	cmd = in.nextLine();
		    	
		    	if ((cmd.equalsIgnoreCase("stop")) || (cmd.equalsIgnoreCase("close")) || (cmd.equalsIgnoreCase("exit")))
		    		break;
		    	else
		    		try{
		    			execute(cmd);
		    		}catch (Throwable e) {
		    			e.printStackTrace();
		    		}
		    }
		
		} finally{
			engine.stop();
		}
	}
	
	public void execute(String executeCmd){
		StringTokenizer st  = new StringTokenizer(executeCmd);
		if (st.hasMoreTokens()){
			String cmd = st.nextToken();
			if  (cmd.startsWith("help")){
				System.out.println(" ======================================= Command List ============================================ ");
				System.out.println(" newInstance <definitionName>                  ");
				System.out.println(" list-def                					   ( List All Process Definitions )                          ");
				System.out.println(" task-for <definition_name!>  				   ( List All Task Definition for a Definition )");
				System.out.println(" list-instances <definition_name?>  		   ( List All Process Instances (For Definition if is Given )");
				System.out.println(" setVar <instanceID!> <varName!> <varValue!>   ( Set a variable in the instance ) ");
				System.out.println(" task-instances-for <instancesId!>  		   ( List All Task Definition for a Definition )");
				System.out.println(" ex-task <instanceID!> <taskInstanceId!>       ( Execute a TaskInstance for a InstanceID ) ");
				
			}else if  (cmd.startsWith("newInstance")){
				String processName = st.nextToken().trim();
				long instanceId = controlAPI.startByProcessName(processName);
			}else if (cmd.startsWith("list-def")){
				org.spagic.workflow.api.Process[] defs = queryAPI.getProcesses();
				
				for (org.spagic.workflow.api.Process d : defs){
					System.out.println("Process ID ["+d.getProcessId()+"] Name ["+d.getName()+"] Version ["+d.getVersion()+"]");
				}
			}else if (cmd.startsWith("list-instances")){
				String processName = null;
				org.spagic.workflow.api.ProcessInstance ins[] = null;
				if (st.hasMoreTokens()){
					processName = st.nextToken();
				}
				
				if (processName == null){
					ins = queryAPI.getProcessInstances();
				}else {
					ins = queryAPI.getProcessInstances(processName);
				}
				
				for (org.spagic.workflow.api.ProcessInstance i : ins){
					System.out.println("Process Instance ["+i.getProcessInstanceId()+"] Name ["+i.getName()+"] State ["+i.getState()+"]");
				}
			} else if (cmd.startsWith("task-instances-for")){
				
				if (st.countTokens() < 1){
					System.out.println(" The setVar command must be in the form setVar <INSTANCE_ID>");
				}
				String instanceID = st.nextToken();
				org.spagic.workflow.api.TaskInstance[] tis = queryAPI.getHumanTaskInstances(Long.valueOf(instanceID));
				for ( org.spagic.workflow.api.TaskInstance taski : tis){
					System.out.println ( " -- TaskInstance ["+taski.getTaskInstanceId()+"] ["+taski.getTaskName()+"] ["+printStatus(taski.getStatus())+"]");
				}
			} else if (cmd.startsWith("task-for")){
				String processName = st.nextToken().trim();
				org.spagic.workflow.api.Task[] taskDef = queryAPI.getHumanTask(processName);
				
				for ( org.spagic.workflow.api.Task task : taskDef){
					
					System.out.println ( " -- Task ["+task.getTaskId()+"] ["+task.getTaskName()+"]");
				}	
			}else if  (cmd.startsWith("setVar")){
				if (st.countTokens() < 3){
					System.out.println(" The setVar command must be in the form setVar <INSTANCE_ID> <VAR_NAME> <VAR_VALUE>");
				}
				String instanceID = st.nextToken();
				String varName = st.nextToken();
				String varValue = st.nextToken();
				controlAPI.setGlobalVariable(Long.valueOf(instanceID), varName, varValue);
			/*}else if (executeCmd.startsWith("signal")){
				if (currentInstance.hasEnded()){
					System.out.println("Process Instance is Ended");
				}else{
					if (currentInstance != null)
						currentInstance.signal();
					else
						System.out.println("No current instance deployed");
					status(currentInstance);
				}*/
			}else if (executeCmd.startsWith("ex-task")){
				if (st.countTokens() < 1){
					System.out.println(" The ex-task command must be in the form ex-task <INSTANCEID> <TASK_ID>");
				}
				String instanceID = st.nextToken();
				String taskId = st.nextToken().trim();
				controlAPI.executeTask(Long.valueOf(instanceID), Long.valueOf(taskId), null);
				
			}else if (executeCmd.startsWith("abort")){
				String instanceID = st.nextToken();
				controlAPI.abort(Long.valueOf(instanceID));
			}else{
				System.out.println("Comando non riconosciuto");
			}
		}
		
		
		
	}
	
	public String printStatus(int i){
		String status = "UNKNOW";
		if (i == 0){
			status = "NOT_GENERATED";
		}
		
		if (i == 1){
			status = "GENERATED";
		}
		
		
		if (i == 2){
			status =  "COMPLETED";
		}
		
		if (i == 3){
			status =  "CANCELLED";
		}
		return status;
	}
	
	public static void main(String[] args) {
		try{
			APITester tester = new APITester();
			tester.open();
		}catch (Exception e) {
			e.printStackTrace();
		}
	}

}
